from typing import Optional
from enum import Enum
from pydantic import BaseModel,  ValidationError

class Role(Enum):
    STUDENT='student'
    TEACHER='teacher'

class Section(BaseModel):
    id:str
    section_num:int
    people:Optional[list[(str,Role)]]=[]

    class Config:
        validate_assignment = True 

class Course(BaseModel):
    id:str
    dept_code:str
    course_num:int
    title:str
    description:Optional[str]="No description provided."
    sections:Optional[list[str]]=[]

    class Config:
        validate_assignment = True    

if __name__=="__main__":

    c = Course(id='cid123',
               dept_code='CS',
               course_num=518,
               title="Intro to SWE")
    
    try: c.sections='asdf'
    except ValidationError: print('invalid assignment (expected)')

    # # appended items will not be checked by default
    # c.sections.append(5)

    s = Section(id='sid123',section_num=1)

    user_id = 'uid123'
    r = Role('student')
    s.people.append((user_id,r))
    print(s)

    c.sections.append(s.id)
    print(c)

    

