from typing import Optional
from pydantic import BaseModel, ValidationError

# embedded data
class Address(BaseModel):
    line1:str
    line2:Optional[str]=None
    city:str
    state:str
    zip:str

class User(BaseModel):
    id:str
    username:str
    password:str
    address:Optional[Address]=None

    class Config:
        validate_assignment = True        

class UserCollection(BaseModel):
    users: list[User]

if __name__=="__main__":
    
    user_dict = {'id':'uid123',
                 'username': 'joe', 
                 'password': 'schmoe'}
    u1 = User(**user_dict)
    print(u1)

    address = {'line1':'123 Academic Way',
                'city':'Durham',
                'state':'NH',
                'zip':'03824'}
    addr = Address(**address)
    print(addr)

    try: u1.address='blah'
    except ValidationError: print('bad address (expected)')
    u1.address=addr

    u2 = User(id='uid456',
              username='jane',
              password='schmane')
    print(u2)

    uc = UserCollection(users=[u1,u2])
    print(uc)

