import pymongo
from bson import ObjectId
from bson.errors import InvalidId
import pymongo.errors

from user_models import *

usersCollection = None

class UserManager:

    def __init__(self, conn_str:str):
        '''connect to db server and set self.col;
        create unique index'''
        global usersCollection

        dbClient = pymongo.MongoClient(conn_str)
        userDatabase = dbClient["user_database"]
        usersCollection = userDatabase["users"]
        usersCollection.create_index("username", unique=True)

    def reset(self):
        ''' drop collection and recreate index'''
        global usersCollection

        # drop collection
        usersCollection.drop()

        # recreate index
        usersCollection.create_index("username", unique=True)


    def create_user(self, user:UserIn) -> CreateUserResult:
        '''create user and return result'''
        global usersCollection

        try:
            newUserId = usersCollection.insert_one(user.model_dump(by_alias=True, exclude=["id"])).inserted_id
            return CreateUserResult(status = 1, inserted_id=str(newUserId))
        except pymongo.errors.DuplicateKeyError:
            return CreateUserResult(status = 0, message="Sorry, username is alredy taken.")
        
        
    def read_users(self,query={}) -> UserCollection:
        ''' get all users'''
        global usersCollection
        userCursor = usersCollection.find(query)
        users=[]
        for user in userCursor:
            # need to conver _id which is of type ObjectId from pymongo
            # to type string for pydantic UserOut model
            user["_id"] =  str(user["_id"])
            users.append(UserOut(**user))
        
        # Return the wrapped users in a UserCollection
        return UserCollection(users=users)
        

    def read_user(self, user_id: str) -> UserOut | None:
        ''' get user by id '''
        global usersCollection
        try:
            oid = ObjectId(user_id)
            userDoc = usersCollection.find_one({"_id": oid})
            if userDoc is None:
                return None
            else:
                # need to conver _id which is of type ObjectId from pymongo
                # to type string for pydantic UserOut model
                userDoc["_id"] =  str(userDoc["_id"])
                return UserOut(**userDoc)  
        except InvalidId:
            return None  # Return None if the ObjectId is invalid

        
                
    
    def update_user(self, user_id: str, updates:dict) -> UpdateUserResult:
        ''' perform update and return result'''
        global usersCollection
        oid = ObjectId(user_id)
        try:
            updateRes = usersCollection.update_one({"_id": oid}, {"$set": updates}).modified_count
            return UpdateUserResult(status = 1, modified_count=updateRes)
        except:
            return UpdateUserResult(status = 0, modified_count=0)
       
    
    def delete_user(self, user_id: str) -> DeleteUserResult:
        global usersCollection
        oid = ObjectId(user_id)
        try:
            delRes = usersCollection.delete_one({"_id": oid})
            return DeleteUserResult(status = 1, delete_count=delRes.deleted_count)
        except:
            return DeleteUserResult(status = 0, delete_count=0)