

import unittest


# Custom Test Result with Emojis
class EmojiTestResult(unittest.TextTestResult):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.passed_tests = 0
        self.failed_tests = 0

    def addSuccess(self, test):
        super().addSuccess(test)
        print("\033[92m✅\033[0m Passed:", test)  # Green checkmark for success
        self.passed_tests += 1

    def addFailure(self, test, err):
        super().addFailure(test, err)
        print("\033[91m❌\033[0m Failed:", test)  # Red cross for failure
        self.failed_tests += 1

    def stopTestRun(self):
        # Roll-up stats after test run
        print("\n")
        print(f"Test Summary: {self.passed_tests} Passed, {self.failed_tests} Failed")
        if self.failed_tests == 0:
            print("🎉 All tests passed!")
        else:
            num_passed = self.passed_tests
            num_failed = self.failed_tests
             # Print the summary with actual counts
            print(f"{num_passed} tests passed, {num_failed} tests failed")

            # List failed tests
            if num_failed > 0:
                print("\nFailed Tests:")
                for test in self.failed_tests:
                    print(f"❌ {test}")

            # List passed tests
            if num_passed > 0:
                print("\nPassed Tests:")
                for test in self.passed_tests:
                    print(f"✅ {test}")

# Custom Test Runner to use the custom TestResult
class EmojiTestRunner(unittest.TextTestRunner):
    def _makeResult(self):
        return EmojiTestResult(self.stream, self.descriptions, self.verbosity)