---
marp: true
---

# API Clients

---
# Clients
* Use service clients to connect to your services

---
# Generic API client
* Methods:
    - create
    - read (read_all, read_by_id, read)
    - update
    - delete_all
    - delete_by_id
* You can use a generic API client if all of your services conform to the same conventions for these methods (e.g. route, method, parameters)

---
# Generic API client
```python
import requests as rq

class APIClient:
    ''' generic API client'''

    def __init__(self,url):
        self.url = url

    def create(self,body):
        res = rq.post(self.url,json=body) 
        res.raise_for_status()  
        return res.json()
```

---
# Using exceptions
* raise_for_status will raise an exception for any responses that do not return a status_code of 200
* you can handle the exception in your flask app, and get information:
```python
        try:
            u = client.read_by_id('invalidUid')
            self.assertTrue(False)    
        except rq.exceptions.HTTPError as e:
            self.assertEqual(e.response.status_code,404)
            # print(e.response.text)   
```

---
# Extending the generic client
* You can add service-specific functionality by extending the generic client
* UserClient extends (inherits from) APIClient
```python
class UserClient(APIClient):

    def __init__(self,url):
        super().__init__(url)

    # You can add other methods that are unique to User here
```