# Containerize and deploy

TOC:
* Containerize with Docker
* Add image to ACR
* Deploy to ACA

## Containerizing your app with Docker

### Initialize your assets

* in your web_app directory (where app.py is located), run command: 
``` 
docker init 
```
* Choose these options:  
    * python  
    * port: 80  
    * command: 
```
flask run --port=80 --host=0.0.0.0 
```
* in your Dockerfile, find and comment out this line:  
```
## Switch to the non-privileged user to run the application.  
#USER appuser  
```
* create requirements.txt and add  
```
flask  
requests  
```

### Run the Application

* build and run:
``` 
docker compose up --build 
```
* verify:  
  * go to the localhost URL given in the terminal  
* stop: 
``` 
docker compose down 
```

## Adding your container image to Azure Container Registry (ACR)

### Create registry and login:

* Naming:  
  * You created a resource group when you deployed your API.  You can find it in the Azure portal.  
  * Choose your own name for the registry.  The name must be unique within the RG.    
* commands:  
```
az acr create \
    --resource-group <RG-NAME> \
    --name <REG-NAME> \
    --sku Basic

az acr login --name <REG-NAME>  
```

### Update your compose.yaml file

* add the following to compose.yaml, under server: 
``` 
image: <REG-NAME>.azurecr.io/<IMAGE-NAME>
```
* example:
```
services:
server:
    build:
    context: .
    ports:
    - 80:80
    image: cs518registry.azurecr.io/web-app-fa24
    platform: linux/amd64
```

### Push to ACR

* use this command:
```
docker compose push
```

## Deploy to Azure Container Apps

* You can use all of the defaults if you want to, you just need to specify your image name.
```
az containerapp up \
  --name my-container-app \
  --resource-group my-container-apps \
  --location centralus \
  --environment 'my-container-apps' \
  --image <IMAGE_NAME> \
  --target-port 80 \
  --ingress external \
  --query properties.configuration.ingress.fqdn
```

* If you run into problems related to permissions for username and password when running the az containerapp command you may just need to add two arguments to the command:

```
--registry-username <containerRegistryUsername> \
--registry-password <password> \

```
* In order to get the registry username and password you will have to generate them in azure.

# Generating a username and password for Azure Container Registry
* Login to the Azure portal : https://portal.azure.com/
* In the search bar searc for 'container registries'

![image](images/container_registry_search.png)

* Click on container registries option in the drop down menu and it will bring you to a list of container registries associated with your azure account.
![image](images/container_registry_list.png)

* Select the container registry you created earlier in the lab.

* You should see the selected container registry's 'overview' page. In the left navigation bar navigate to Settings->Access Keys

![image](images/container_registry_settings.png)

* In the Access Keys view click the checkbox 'Admin User' this will then generate the username and password

![image](images/container_registry_admin.png)

* You can now copy the username and password from the Access Keys view:

![image](images/container_registry_username_pass.png)

* Now you can rerun the az containerapp command with the added registry-username and registry-password arguments.

* Example:
```
az containerapp up \
  --name my-container-app \
  --resource-group my-container-apps \
  --location centralus \
  --environment 'my-container-apps' \
  --image <IMAGE_NAME> \
  --target-port 80 \
  --ingress external \
  --query properties.configuration.ingress.fqdn
  --registry-username <containerRegistryUsername> \
  --registry-password <password> \
```

### References

* https://docs.docker.com/guides/python/containerize/
* https://learn.microsoft.com/en-us/azure/container-registry/container-registry-get-started-azure-cli
* https://learn.microsoft.com/en-us/azure/container-apps/get-started 