# Deploying your App

### Overview

* In this lab, you will put your app into a Docker container and deploy to Azure Container Instances (ACI)
* You will:
  1. create a registry in Azure Container Registries (ACR)
  2. create your container
  3. build and run your container locally
  4. push your container image to ACR
  5. deploy your container to ACI.  
* ACI will pull the container image from ACR

### **Create registry and login:**

* Naming:  
  * You created a resource group when you deployed your API.  You can find it in the Azure portal.  
  * Choose your own name for the registry.  The name must be unique RG.    
* commands:  
```
az acr create --resource-group <RG-NAME> --name <REG-NAME> --sku Basic

az acr login --name <REG-NAME>  
```
* example:  
```
az acr create --resource-group cs518-rg --name cs518registry --sku Basic

az acr login --name cs518registry
```

### Create your container

* in your web\_app directory (where app.py is located), run command: 
  * ``` docker init ```
* Choose these options:  
    * python  
    * port: 80  
    * command: ```flask run --port=80 --host=0.0.0.0 ```
* in your Dockerfile, find and comment out this line:  
```
## Switch to the non-privileged user to run the application.  
#USER appuser  
```
* create requirements.txt and add  
```
flask  
requests  
```
* add to compose.yaml : 
  * ``` image: <REG-NAME>.azurecr.io/<IMAGE-NAME>```
* example:
  ```
  services:
    server:
      build:
        context: .
      ports:
        - 80:80
      image: cs518registry.azurecr.io/web-app-fa24
      platform: linux/amd64
  ```

### **build and run:**

* build and run: 
  *  ``` docker compose up --build ```
* verify:  
  * go to the localhost URL given in the terminal  
* stop:  
  * ``` docker compose down ```

### **push to ACR and list images**

* commands:  
```
docker push <REG-NAME>.azurecr.io/<IMAGE-NAME>

az acr repository list --name <REG-NAME> --output table
```
* example:  
```
docker push cs518registry.azurecr.io/web-app-fa24

az acr repository list --name cs518registry --output table
```

### **deploy**

* naming:  
  * resource group, registry, and image are named above.  
  * container name and dns-name-label can be the same, but they don't have to be.   
* command:  
```
az container create --resource-group <RG-NAME> --name <CONTAINER-NAME> --image <REG-NAME>.azurecr.io/<IMAGE-NAME> --dns-name-label <CONTAINER-NAME> --ports 80
```
* Example:  
```
az container create --resource-group cs518-rg --name cs518-web-app --image cs518registry.azurecr.io/web-app-fa24 --dns-name-label cs518-web-app --ports 80
```
* you may need to enter your registry username and password, which can be found on the Azure portal:  
  * Search bar > Container registries > [Choose your registry] > Settings > Access Keys
  * Note: The password will not be visible after you paste it 

### **Redeploying**

* Rebuild, push, and restart your container group  
```
docker compose build  

docker compose push  
```
* restart container  
```
az container restart --name cs518-web-app --resource-group cs518-rg
```

### **references:**

* [https://docs.docker.com/guides/python/containerize/](https://docs.docker.com/guides/python/containerize/)   
* [https://learn.microsoft.com/en-us/azure/container-registry/container-registry-get-started-azure-cli](https://learn.microsoft.com/en-us/azure/container-registry/container-registry-get-started-azure-cli)   
* [https://learn.microsoft.com/en-us/azure/container-instances/container-instances-quickstart](https://learn.microsoft.com/en-us/azure/container-instances/container-instances-quickstart)
